/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CommProtocolTemplate.h
 *       A implementation of the communication protocol interface.
 *
 *  This implementation may be used to create the base structure of other
 *  communication protocols.
 *
 * @author Tobias Hilbrich
 */

#include "I_CommProtocol.h"
#include "I_Module.h"
#include "ModuleBase.h"

#ifndef COMM_PROTOCOL_TEMPLATE_H
#define COMM_PROTOCOL_TEMPLATE_H

namespace gti
{
    /**
     * Class that describes the communication protocol interface.
     */
    class CommProtocolTemplate : public ModuleBase<CommProtocolTemplate, I_CommProtocol>
    {
    public:

        /**
         * Constructor.
         * @ref ModConf - The module configuration syntax
         * @param intanceName name of the module instance.
         */
        CommProtocolTemplate (const char* instanceName);

        /**
         * Destructor.
         */
        ~CommProtocolTemplate (void);

        /**
         * @see gti::I_CommProtocol::isConnected
         */
        bool isConnected (void);

        /**
         * @see gti::I_CommProtocol::isInitialized
         */
        bool isInitialized (void);

        /**
         * @see gti::I_CommProtocol::isFinalized
         */
        bool isFinalized (void);

        /**
         * @see gti::I_CommProtocol::getNumChannels
         */
        GTI_RETURN getNumChannels (unsigned long* out_numChannels);

        /**
         * @see gti::I_CommProtocol::getPlaceId
         */
        GTI_RETURN getPlaceId (unsigned long* outPlaceId);

        /**
         * @see gti::I_CommProtocol::shutdown
         */
        GTI_RETURN shutdown (void);

        /**
         * @see gti::I_CommProtocol::removeOutstandingRequests
         */
        GTI_RETURN removeOutstandingRequests (void);

        /**
         * @see gti::I_CommProtocol::ssend
         */
        GTI_RETURN ssend (
                void* buf,
                unsigned long num_bytes,
                unsigned long channel
                );

        /**
         * @see gti::I_CommProtocol::isend
         */
        GTI_RETURN isend (
                void* buf,
                unsigned long num_bytes,
                unsigned int *out_request,
                unsigned long channel
                );

        /**
         * @see gti::I_CommProtocol::recv
         */
        GTI_RETURN recv (
                void* out_buf,
                unsigned long num_bytes,
                unsigned long* out_length,
                unsigned long channel,
                unsigned long *out_channel
                );

        /**
         * @see gti::I_CommProtocol::irecv
         */
        GTI_RETURN irecv (
                void* out_buf,
                unsigned long num_bytes,
                unsigned int *out_request,
                unsigned long channel
                );

        /**
         * @see gti::I_CommProtocol::test_msg
         */
        GTI_RETURN test_msg (
                unsigned int request,
                int* out_completed,
                unsigned long* out_receive_length,
                unsigned long* out_channel
                );

        /**
         * @see gti::I_CommProtocol::wait_msg
         */
        GTI_RETURN wait_msg (
                unsigned int request,
                unsigned long* out_receive_length,
                unsigned long* out_channel
                );
    }; /*class CommProtocolTemplate*/
} /*namespace gti*/

#endif /* COMM_PROTOCOL_TEMPLATE_H */
