/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CommStrategyDownTemplate.h
 *       Implementation for the I_CommStrategyDown interface.
 *
 *  This implementation may be used as a template for other implementations.
 *
 * @author Tobias Hilbrich
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"
#include "I_CommStrategyDown.h"
#include "I_CommProtocol.h"
#include "I_Module.h"
#include "ModuleBase.h"

#ifndef COMM_STRATEGY_DOWN_TEMPLATE_H
#define COMM_STRATEGY_DOWN_TEMPLATE_H

namespace gti
{
    /**
     * Class that describes the communication protocol interface.
     */
    class CommStrategyDownTemplate : public ModuleBase<CommStrategyDownTemplate, I_CommStrategyDown>
    {
    protected:
        /** The communication protocol. */
        I_CommProtocol *protocol;

    public:

        /**
         * Constructor.
         * @ref ModConf see for details on module configuration.
         * @param instanceName name of this module.
         */
        CommStrategyDownTemplate (const char* instanceName);

        /**
         *Destructor.
         */
        ~CommStrategyDownTemplate (void);

        /**
         * @see I_CommStrategyDown::shutdown
         */
        GTI_RETURN shutdown (
                GTI_FLUSH_TYPE flush_behavior,
                GTI_SYNC_TYPE sync_behavior);

        /**
         * @see I_CommStrategyDown::getNumClients
         */
        GTI_RETURN getNumClients (unsigned long *outNumClients);

        /**
         * @see I_CommStrategyDown::broadcast
         */
        GTI_RETURN broadcast (
                void* buf,
                unsigned long num_bytes,
                void* free_data,
                GTI_RETURN (*buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyDown::test
         */
        GTI_RETURN test (
                int* out_flag,
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf),
                unsigned long *outChannel,
                unsigned long preferChannel
                );

        /**
         * @see I_CommStrategyDown::wait
         */
        GTI_RETURN wait (
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_fee_function) (void* free_data, unsigned long num_bytes, void* buf),
                unsigned long *outChannel
                );

        /**
         * @see I_CommStrategyDown::acknowledge
         */
        GTI_RETURN acknowledge (
        		unsigned long channel
        		);

        /**
         * @see I_CommStrategyDown::flush
         */
        GTI_RETURN flush (
                void
                );
    };
}

#endif /* COMM_PROTOCOL_TEMPLATE_H */
