/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file I_CommStrategyUp.h
 *        Implementation for the I_CommStrategyUp interface.
 *
 *  This implementation may be used as a template for other implementations.
 *
 * @author Tobias Hilbrich
 */

#include "GtiDefines.h"
#include "GtiEnums.h"
#include "GtiTypes.h"
#include "I_CommStrategyUp.h"
#include "I_CommProtocol.h"
#include "I_Module.h"
#include "ModuleBase.h"

#ifndef COMM_STRATEGY_UP_TEMPLATE_H
#define COMM_STRATEGY_UP_TEMPLATE_H

namespace gti
{
    /**
     * Class that describes the communication protocol interface.
     */
    class CommStrategyUpTemplate : public ModuleBase<CommStrategyUpTemplate, I_CommStrategyUp>
    {
    protected:
        /** The communication protocol. */
        I_CommProtocol *protocol;

    public:

        /**
         * Constructor.
         * @ref ModConf see for details on module configuration.
         * @param instanceName name of this module.
         */
        CommStrategyUpTemplate (const char* instanceName);

        /**
         * Destructor.
         */
        ~CommStrategyUpTemplate (void);

        /**
         * @see I_CommStrategyUp::shutdown
         */
        GTI_RETURN shutdown (
                GTI_FLUSH_TYPE flush_behavior,
                GTI_SYNC_TYPE sync_behavior);

        /**
         * @see I_CommStrategyUp::send
         */
        GTI_RETURN send (
                void* buf,
                unsigned long num_bytes,
                void* free_data,
                GTI_RETURN (*buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::test
         */
        GTI_RETURN test (
                int* out_flag,
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::wait
         */
        GTI_RETURN wait (
                unsigned long* out_num_bytes,
                void** out_buf,
                void** out_free_data,
                GTI_RETURN (**out_buf_free_function) (void* free_data, unsigned long num_bytes, void* buf)
                );

        /**
         * @see I_CommStrategyUp::raisePanic
         */
        GTI_RETURN raisePanic (
                void
        );

        /**
         * @see I_PanicListener::flush
         */
        GTI_RETURN flush (
                void
                );

        /**
         * @see I_PanicListener::flushAndSetImmediate
         */
        GTI_RETURN flushAndSetImmediate (void);
    };
}

#endif /* COMM_STRATEGY_UP_TEMPLATE_H */
