/*
 * GTI (Generic Tools Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2011-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2011-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file PanicReceiver.h
 *       @see MUST::PanicReceiver.
 *
 *  @date 11.07.2012
 *  @author Tobias Hilbrich, Joachim Protze, Mathias Korepkat
 */

#include "ModuleBase.h"
#include "I_PanicListener.h"

#include "I_PanicReceiver.h"

#include <list>

#ifndef PANICRECEIVER_H
#define PANICRECEIVER_H

namespace gti
{
    /**
     * Implementation of I_PanicReceiver.
     */
    class PanicReceiver : public ModuleBase<PanicReceiver, I_PanicReceiver>
    {
    public:
        /**
         * Constructor.
         * @param instanceName name of this module instance.
         */
        PanicReceiver (const char* instanceName);

        /**
         * Destructor.
         */
        virtual ~PanicReceiver (void);

        /**
         * @see I_PanicReceiver::notifyPanic
         */
        GTI_ANALYSIS_RETURN notifyPanic (void);

        /**
         * @see I_PanicReceiver::notifyFlush
         */
        GTI_ANALYSIS_RETURN notifyFlush (void);

        /**
         * @see I_PanicReceiver::notifyRaisePanic
         */
        GTI_ANALYSIS_RETURN notifyRaisePanic (void);

    protected:
        std::list<I_PanicListener*> myListeners;
    };
} /*namespace gti*/

#endif /*PANICRECEIVER_H*/
