/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file ThreadedAppStartup.cpp
 * Starts the Tool thread which then executes an instance of
 * ThreadedAppPlace
 *
 *
 * @author Felix Münchhalfen
 */

#include <assert.h>
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <fstream>
#include <sys/time.h>

#include <pnmpimod.h>
#include <dlfcn.h>
#include "ThreadedAppStartup.h"
#include "GtiMacros.h"

static pthread_t m_Thread;
static pthread_once_t thread_starter = PTHREAD_ONCE_INIT; 

void start_worker_thread(void)
{
    std::cout << "Creating tool thread.. " << getpid() << std::endl;
    pthread_create( &(m_Thread), NULL, threadProc, NULL );
    sleep(1);
}

void start() {
    pthread_once( &thread_starter, start_worker_thread );
}

void finish() {
    pthread_join( m_Thread, NULL );
}

void *threadProc( void *thread_data )
{
    // this should be executed immediately after the protocol has been
    // instanciated for a thread; make the first entry for the tool-thread!
    
    // Put this thread on layer 2 for now
    char buf[] = "level_1";
    PNMPI_modHandle_t stack;
    int err=PNMPI_Service_GetStackByName(buf,&stack);
    assert (err==PNMPI_SUCCESS);
    
    int previous_stack = PNMPI_Change_Stack_Explicit(stack);
    
    // Resolve handlePlaceStartup from the .so
    void (*ThreadPlaceStartup) (void) = NULL;
    void (*initHookNotify) (void) = NULL;
#if 0
    //  this does not seem to work? (why?))
    ThreadPlaceStartup = (void (*)(void))(dlsym(RTLD_NEXT, "handlePlaceStartup"));
#else
    void *handle = dlopen ("libthreadedAppPlace.so", RTLD_LAZY);
    assert(handle != NULL);
    ThreadPlaceStartup = (void (*)(void))(dlsym(handle, "handlePlaceStartup"));
    
    handle = dlopen ("libcProtMpiSplited.so", RTLD_LAZY);
    assert(handle != NULL);
    initHookNotify = (void (*)(void))(dlsym(handle, "initHookNotify"));
#endif
    // Startup of the thread-placement-driver
    assert(ThreadPlaceStartup != NULL);
    assert(initHookNotify != NULL);
    initHookNotify();
    ThreadPlaceStartup();
    
    return 0;
}

//=============================
// MPI_Init Wrapper
//=============================
extern "C" int MPI_Init (int* argc, char*** argv)
{
    //Startup the place
    start_worker_thread ();
    
    int err = PMPI_Init (argc, argv);

    return err;
}

//=============================
// MPI_Init_thread Wrapper
//=============================
extern "C" int MPI_Init_thread (int* argc, char*** argv, int required, int* provided)
{
    //Startup the place
    start_worker_thread ();    
    
    int err = PMPI_Init_thread (argc, argv, required, provided);

    return err;
}

/*EOF*/
