/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2009-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2009-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file SimpleTcpServer.h
 * A TCP server that establishes connections between
 * processes. Uses the protocol from cProtSimpleTcp.
 * @see CProtSimpleTCP
 * @author Tobias Hilbrich
 * @date 23.04.2009
 */

#ifndef SIMPLE_TCP_SERVER_H
#define SIMPLE_TCP_SERVER_H

#include <map>
#include <list>
#include <unistd.h>

/**
 * A participant of a connection.
 */
class Participant
{
public:
    unsigned int    mySide;
    unsigned long   myTierID;
    unsigned long   mySelfID;
    unsigned long   myIP;
    unsigned long   myPort;
    unsigned long   myTierSize;
    int             mySocket;

    Participant (void);
    Participant (unsigned int side, unsigned long tierID,
                 unsigned long selfID, unsigned long tierSize,
                 unsigned long   IP, unsigned long port,
                 int socket);
    void closeSocket (void);
    void printParticipant (void);
};

/**
 * Class for a partition of the connection being established.
 */
class Bundle
{
public:
    Participant             myParticipantT;
    std::list<Participant>  myParticipantsB;

    Bundle (Participant t, std::list<Participant> bList);
};


/**
 * A connection layer.
 */
class Tier
{
protected:
    bool bTierSizeTKnown;
    bool bTierSizeBKnown;

    bool checkForTierCompletness (void);
    std::list<Bundle> partitionTier (void);
    void sendBundles (std::list<Bundle> bundles);
    void printBundles (std::list<Bundle> bundles);

public:
    unsigned long myTierID;
    unsigned long myTierSizeT;
    unsigned long myTierSizeB;
    std::map <unsigned long, Participant> myParticipantsT;
    std::map <unsigned long, Participant> myParticipantsB;
    static bool                mySetConnectPort;
    static unsigned long       myConnectPort;

    Tier (unsigned long myTierID, unsigned long connectPort);
    ~Tier (void);
    /**
     * @return true if this was the last participant of
     *         this tier and the communication between
     *         the tier members was established,
     *         false otherwise.
     */
    bool addParticipant (Participant newParticipant);
    void printTier (void);
};

/**
 * The TCP connection server.
 */
class Server
{
protected:
    std::map <unsigned long, Tier> myTiers;
    int                 myServerSocket;
    unsigned long       myIP;
    unsigned long       myPort;
    unsigned long       myConnectPort;

    void setupServerSocket (void);
    void acceptConnections (void);

public:
    bool                myTerminate;

    Server (unsigned long IP, unsigned long listenPort, unsigned long connectPort);
    ~Server (void);
};

#endif /*SIMPLE_TCP_SERVER_H*/
