/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CallIdInput.cpp
 * 		@see gti::weaver::CallIdInput
 *
 * @author Tobias Hilbrich
 * @date 07.01.2011
 */

#include "CallIdInput.h"
#include <stdio.h>

using namespace gti::weaver::calls;

//=============================
// CallIdInput
//=============================
CallIdInput::CallIdInput ( )
 : myTargetCall (NULL)
{

}

//=============================
// CallIdInput
//=============================
CallIdInput::CallIdInput (Call* call)
 : myTargetCall (call)
{

}

//=============================
// ~CallIdInput
//=============================
CallIdInput::~CallIdInput ( )
{
	myTargetCall = NULL;
}

//=============================
// getCallId
//=============================
int CallIdInput::getCallId (void) const
{
	return myTargetCall->getUniqueId ();
}

//=============================
// getName
//=============================
std::string CallIdInput::getName ( ) const
{
	char temp[128];
	sprintf (temp, "%d", getCallId());
	return temp;
}

//=============================
// getType
//=============================
std::string CallIdInput::getType ( ) const
{
	return "int";
}

//=============================
// print
//=============================
std::ostream& CallIdInput::print (std::ostream& out) const
{
	out << "callIdInput{";
	Input::print(out);
	out << "}";

	return out;
}

//=============================
// getDotInputNodeName
//=============================
std::string CallIdInput::getDotInputNodeName (std::string callName)
{
	return callName + ":" + callName;
}

/*EOF*/
