/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file CallNameInput.cpp
 * 		@see gti::weaver::CallNameInput
 *
 * @author Tobias Hilbrich
 * @date 03.08.2010
 */

#include "CallNameInput.h"
#include <stdio.h>

using namespace gti::weaver::calls;

//=============================
// CallNameInput
//=============================
CallNameInput::CallNameInput ( )
 : myTargetCall (NULL)
{

}

//=============================
// CallNameInput
//=============================
CallNameInput::CallNameInput (Call* call)
 : myTargetCall (call)
{

}

//=============================
// ~CallNameInput
//=============================
CallNameInput::~CallNameInput ( )
{
	myTargetCall = NULL;
}

//=============================
// getCallName
//=============================
std::string CallNameInput::getCallName (void)
{
	return myTargetCall->getName();
}

//=============================
// getName
//=============================
std::string CallNameInput::getName ( ) const
{
	return "\"" + myTargetCall->getName() + "\"";
}

//=============================
// getType
//=============================
std::string CallNameInput::getType ( ) const
{
	return "char*"; /**< removed "const ", I think we don't need it*/
}

//=============================
// isArrayInput
//=============================
bool CallNameInput::isArrayInput () const
{
	return true;
}

//=============================
// getLenName
//=============================
std::string CallNameInput::getLenName () const
{
	char temp[64];
	sprintf (temp, "%ld", getName().length());
	return temp;
}

//=============================
// print
//=============================
std::ostream& CallNameInput::print (std::ostream& out) const
{
	out << "callNameInput{";
	Input::print(out);
	out << "}";

	return out;
}

//=============================
// getDotInputNodeName
//=============================
std::string CallNameInput::getDotInputNodeName (std::string callName)
{
	return callName + ":" + callName;
}

/*EOF*/
