/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file RequiresApi.cpp
 * 		@see gti::weaver::RequiresApi
 *
 * @author Tobias Hilbrich
 * @date 14.01.2010
 */

#include <assert.h>
#include <iostream>

#include "RequiresApi.h"

using namespace gti::weaver::modules;

//=============================
// RequiresApi
//=============================
RequiresApi::RequiresApi ()
	: myApis()
{

}

//=============================
// RequiresApi
//=============================
RequiresApi::RequiresApi (std::list<std::string> apis)
	: myApis()
{
	std::list<std::string>::iterator i;
	for (i = apis.begin(); i != apis.end(); i++)
	{
		myApis.push_back (*i);
	}
}

//=============================
// ~RequiresApi
//=============================
RequiresApi::~RequiresApi ( )
{

}

//=============================
// addApi
//=============================
void RequiresApi::addApi (std::string api)
{
	myApis.push_back (api);
}

//=============================
// removeApi
//=============================
void RequiresApi::removeApi (std::string api)
{
	std::list<std::string>::iterator i;
	for (i = myApis.begin(); i != myApis.end(); i++)
	{
		if (*i == api)
		{
			myApis.erase(i);
			break;
		}
	}
}

//=============================
// getRequiredApis
//=============================
std::list<std::string> RequiresApi::getRequiredApis (void)
{
	return myApis;
}

//=============================
// print
//=============================
std::ostream& RequiresApi::print (std::ostream& out) const
{
	if (myApis.size() > 0)
	{
		out << "requiredAPIs={";
		for (std::list<std::string>::const_iterator i = myApis.begin();
			 i != myApis.end(); i++)
		{
			if (i != myApis.begin())
				out << ", ";

			out << *i;
		}
		out << "}";
	}
	else
	{
		out << "noRequiredAPIs";
	}
	return out;
}

/*EOF*/
