/*
 * GTI (Generic Tool Infrastructure)
 * Copyright (C) ZIH, Technische Universitaet Dresden, Federal Republic of Germany, 2010-2015
 * Copyright (C) Lawrence Livermore National Laboratories, United States of America, 2010-2015
 * Copyright (C) RWTH Aachen University, Federal Republic of Germany, 2013-2015
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file Adjacency.cpp
 * 		@see gti::weaver::Adjacency
 *
 * @author Tobias Hilbrich
 * @date 13.01.2010
 */

#include "Adjacency.h"

using namespace gti::weaver::layout;

//=============================
// Adjacency
//=============================
Adjacency::Adjacency (
        Level* target,
        Communication* comm,
        DistributionType distrib,
        int blocksize)
:  target (target),
   comm (comm),
   distrib (distrib),
   blocksize (blocksize)
{

}

//=============================
// Adjacency
//=============================
Adjacency::Adjacency ()
: target (NULL),
  comm (NULL),
  distrib (DISTRIBUTION_UNIFORM),
  blocksize (0)
{

}

//=============================
// ~Adjacency
//=============================
Adjacency::~Adjacency ()
{
	/*Do not free the pointers,
	 *their storage is managed by
	 *their respective singletons.
	 */
	target = NULL;
	comm = NULL;
}

//=============================
// getTarget
//=============================
Level* Adjacency::getTarget (void)
{
	return target;
}

//=============================
// getComm
//=============================
Communication* Adjacency::getComm (void)
{
	return comm;
}

//=============================
// print
//=============================
std::ostream& Adjacency::print (std::ostream& out) const
{
	if (!target || !comm)
	{
		out << "invalidAdjacency";
		return out;
	}

	out
		<< "adjacency={targetLevel="
		<< target->getOrder()
		<< ", "
		<< *comm
		<< "}";

	return out;
}

//=============================
// getDistribution
//=============================
DistributionType Adjacency::getDistribution (void)
{
    return distrib;
}

//=============================
// getBlocksize
//=============================
int Adjacency::getBlocksize (void)
{
    return blocksize;
}

/*EOF*/
